@@ -13,6 +13,8 @@ from .core.sets import set_constructor as _set_constructor
 from .core.utils import (to_unique_sorted_items as _to_unique_sorted_items,
                          to_unique_sorted_values as _to_unique_sorted_values)
 from .hints import (Key,
+                    MapFactory,
+                    SetFactory,
                     Value)
 
 Node = Node
@@ -104,60 +106,49 @@ class Tree(_Tree[Key, Value]):
 
     def max(self) -> Node:
         node = self.root
-        if node is NIL:
-            raise ValueError('Tree is empty.')
-        while node.right is not NIL:
-            node = node.right
-        self._splay(node.key)
+        if node is not NIL:
+            while node.right is not NIL:
+                node = node.right
+            self._splay(node.key)
         return node
 
     def min(self) -> Node:
         node = self.root
-        if node is NIL:
-            raise ValueError('Tree is empty.')
-        while node.left is not NIL:
-            node = node.left
-        self._splay(node.key)
-        return node
-
-    def next(self, key: Key) -> Node:
-        node = _Tree.next(self, key)
-        self._splay(node.key)
+        if node is not NIL:
+            while node.left is not NIL:
+                node = node.left
+            self._splay(node.key)
         return node
 
     def popmax(self) -> Node:
         if self.root is NIL:
-            raise KeyError
+            return self.root
         result = self.max()
         self._remove_root()
         return result
 
     def popmin(self) -> Node:
         if self.root is NIL:
-            raise KeyError
+            return self.root
         result = self.min()
         self._remove_root()
         return result
 
     def predecessor(self, node: Node) -> AnyNode:
         if node.left is NIL:
-            candidate, cursor, key = NIL, self.root, node.key
+            result, cursor, key = NIL, self.root, node.key
             while cursor is not node:
                 if cursor.key < key:
-                    candidate, cursor = cursor, cursor.right
+                    result, cursor = cursor, cursor.right
                 else:
                     cursor = cursor.left
-            return candidate
         else:
             result = node.left
             while result.right is not NIL:
                 result = result.right
-            return result
-
-    def prev(self, key: Key) -> Node:
-        node = _Tree.prev(self, key)
-        self._splay(node.key)
-        return node
+        if result is not NIL:
+            self._splay(result.key)
+        return result
 
     def remove(self, node: Node) -> None:
         self._splay(node.key)
@@ -165,18 +156,19 @@ class Tree(_Tree[Key, Value]):
 
     def successor(self, node: Node) -> AnyNode:
         if node.right is NIL:
-            candidate, cursor, key = NIL, self.root, node.key
+            result, cursor, key = NIL, self.root, node.key
             while cursor is not node:
                 if key < cursor.key:
-                    candidate, cursor = cursor, cursor.left
+                    result, cursor = cursor, cursor.left
                 else:
                     cursor = cursor.right
-            return candidate
         else:
             result = node.right
             while result.left is not NIL:
                 result = result.left
-            return result
+        if result is not NIL:
+            self._splay(result.key)
+        return result
 
     def _splay(self, key: Key) -> None:
         next_root = self.root
@@ -230,5 +222,5 @@ class Tree(_Tree[Key, Value]):
         return replacement
 
 
-map_ = partial(_map_constructor, Tree.from_components)
-set_ = partial(_set_constructor, Tree.from_components)
+map_ = partial(_map_constructor, Tree.from_components)  # type: MapFactory
+set_ = partial(_set_constructor, Tree.from_components)  # type: SetFactory
